/*
 * Copyright 2002-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.context.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.annotation.Inherited;

/**
 * Indicates whether a bean is to be lazily initialized.
 *
 * <p>May be used on any class directly or indirectly annotated with
 * {@link org.springframework.stereotype.Component} or on methods annotated with
 * {@link Bean}.
 *
 * <p>If this annotation is not present on a Component or Bean definition, eager
 * initialization will occur. If present and set to {@code true}, the
 * Bean/Component will not be initialized until referenced by another bean or
 * explicitly retrieved from the enclosing
 * {@link org.springframework.beans.factory.BeanFactory}. If present and set to
 * {@code false}, the bean will be instantiated on startup by bean factories
 * that perform eager initialization of singletons.
 *
 * <p>If Lazy is present on a {@link Configuration @Configuration} class, this
 * indicates that all {@link Bean @Bean} methods within that {@code @Configuration}
 * should be lazily initialized. If Lazy is present and false on a Bean method
 * within a Lazy-annotated Configuration class, this indicates overriding the
 * 'default lazy' behavior and that the bean should be eagerly initialized.
 *
 * @author Chris Beams
 * @since 3.0
 * @see Primary
 * @see Bean
 * @see Configuration
 * @see org.springframework.stereotype.Component
 */
@Target({ElementType.TYPE, ElementType.METHOD})
@Retention(RetentionPolicy.RUNTIME)
@Inherited
@Documented
public @interface Lazy {

	/**
	 * Whether lazy initialization should occur.
	 */
	boolean value() default true;

}
