/*
 * Copyright 2002-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.web.method.support;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;

import org.springframework.core.GenericTypeResolver;
import org.springframework.core.LocalVariableTableParameterNameDiscoverer;
import org.springframework.core.MethodParameter;
import org.springframework.core.ParameterNameDiscoverer;
import org.springframework.util.ReflectionUtils;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.HandlerMethod;

/**
 * Provides a method for invoking the handler method for a given request after resolving its method argument 
 * values through registered {@link HandlerMethodArgumentResolver}s.
 * 
 * <p>Argument resolution often requires a {@link WebDataBinder} for data binding or for type conversion. 
 * Use the {@link #setDataBinderFactory(WebDataBinderFactory)} property to supply a binder factory to pass to
 * argument resolvers. 
 * 
 * <p>Use {@link #setHandlerMethodArgumentResolvers(HandlerMethodArgumentResolverComposite)} to customize 
 * the list of argument resolvers.
 *  
 * @author Rossen Stoyanchev
 * @since 3.1
 */
public class InvocableHandlerMethod extends HandlerMethod {

	private HandlerMethodArgumentResolverComposite argumentResolvers = new HandlerMethodArgumentResolverComposite();

	private WebDataBinderFactory dataBinderFactory;

	private ParameterNameDiscoverer parameterNameDiscoverer = new LocalVariableTableParameterNameDiscoverer();

	/**
	 * Constructs a new handler method with the given bean instance and method.
	 * @param bean the bean instance
	 * @param method the method
	 */
	public InvocableHandlerMethod(Object bean, Method method) {
		super(bean, method);
	}

	/**
	 * Constructs a new handler method with the given bean instance, method name and parameters.
	 * @param bean the object bean
	 * @param methodName the method name
	 * @param parameterTypes the method parameter types
	 * @throws NoSuchMethodException when the method cannot be found
	 */
	public InvocableHandlerMethod(
			Object bean, String methodName, Class<?>... parameterTypes) throws NoSuchMethodException {
		super(bean, methodName, parameterTypes);
	}

	/**
	 * Sets the {@link WebDataBinderFactory} to be passed to argument resolvers allowing them to create
	 * a {@link WebDataBinder} for data binding and type conversion purposes.
	 * @param dataBinderFactory the data binder factory.
	 */
	public void setDataBinderFactory(WebDataBinderFactory dataBinderFactory) {
		this.dataBinderFactory = dataBinderFactory;
	}

	/**
	 * Set {@link HandlerMethodArgumentResolver}s to use to use for resolving method argument values.
	 */
	public void setHandlerMethodArgumentResolvers(HandlerMethodArgumentResolverComposite argumentResolvers) {
		this.argumentResolvers = argumentResolvers;
	}

	/**
	 * Set the ParameterNameDiscoverer for resolving parameter names when needed (e.g. default request attribute name).
	 * <p>Default is an {@link org.springframework.core.LocalVariableTableParameterNameDiscoverer} instance.
	 */
	public void setParameterNameDiscoverer(ParameterNameDiscoverer parameterNameDiscoverer) {
		this.parameterNameDiscoverer = parameterNameDiscoverer;
	}

	/**
	 * Invoke the method after resolving its argument values in the context of the given request. <p>Argument 
	 * values are commonly resolved through {@link HandlerMethodArgumentResolver}s. The {@code provideArgs} 
	 * parameter however may supply argument values to be used directly, i.e. without argument resolution. 
	 * Examples of provided argument values include a {@link WebDataBinder}, a {@link SessionStatus}, or 
	 * a thrown exception instance. Provided argument values are checked before argument resolvers.
	 * 
	 * @param request the current request
	 * @param mavContainer the {@link ModelAndViewContainer} for the current request
	 * @param providedArgs argument values to try to use without view resolution
	 * @return the raw value returned by the invoked method
	 * @exception Exception raised if no suitable argument resolver can be found, or the method raised an exception 
	 */
	public final Object invokeForRequest(NativeWebRequest request, 
										 ModelAndViewContainer mavContainer, 
										 Object... providedArgs) throws Exception {
		Object[] args = getMethodArgumentValues(request, mavContainer, providedArgs);

		if (logger.isTraceEnabled()) {
			StringBuilder builder = new StringBuilder("Invoking [");
			builder.append(this.getMethod().getName()).append("] method with arguments ");
			builder.append(Arrays.asList(args));
			logger.trace(builder.toString());
		}

		Object returnValue = invoke(args);

		if (logger.isTraceEnabled()) {
			logger.trace("Method [" + this.getMethod().getName() + "] returned [" + returnValue + "]");
		}

		return returnValue;
	}

	/**
	 * Get the method argument values for the current request.
	 */
	private Object[] getMethodArgumentValues(NativeWebRequest request, 
											 ModelAndViewContainer mavContainer, 
											 Object... providedArgs) throws Exception {
		MethodParameter[] parameters = getMethodParameters();
		Object[] args = new Object[parameters.length];
		for (int i = 0; i < parameters.length; i++) {
			MethodParameter parameter = parameters[i];
			parameter.initParameterNameDiscovery(this.parameterNameDiscoverer);
			GenericTypeResolver.resolveParameterType(parameter, getBean().getClass());

			args[i] = resolveProvidedArgument(parameter, providedArgs);
			if (args[i] != null) {
				continue;
			}
			if (this.argumentResolvers.supportsParameter(parameter)) {
				args[i] = this.argumentResolvers.resolveArgument(parameter, mavContainer, request, dataBinderFactory);
			}
			else {
				throw new IllegalStateException("Cannot resolve argument index=" + parameter.getParameterIndex() + ""
						+ ", name=" + parameter.getParameterName() + ", type=" + parameter.getParameterType()
						+ " in method " + toString());
			}
		}
		return args;
	}

	/**
	 * Attempt to resolve a method parameter from the list of provided argument values.
	 */
	private Object resolveProvidedArgument(MethodParameter parameter, Object... providedArgs) {
		if (providedArgs == null) {
			return null;
		}
		for (Object providedArg : providedArgs) {
			if (parameter.getParameterType().isInstance(providedArg)) {
				return providedArg;
			}
		}
		return null;
	}
	
	/**
	 * Invoke this handler method with the given argument values.
	 */
	private Object invoke(Object... args) throws Exception {
		ReflectionUtils.makeAccessible(this.getBridgedMethod());
		try {
			return getBridgedMethod().invoke(getBean(), args);
		}
		catch (IllegalArgumentException ex) {
			handleIllegalArgumentException(ex, args);
			throw ex;
		}
		catch (InvocationTargetException ex) {
			handleInvocationTargetException(ex);
			throw new IllegalStateException(
					"Unexpected exception thrown by method - " + ex.getTargetException().getClass().getName() + ": " +
							ex.getTargetException().getMessage());
		}
	}

	private void handleIllegalArgumentException(IllegalArgumentException ex, Object... args) {
		StringBuilder builder = new StringBuilder(ex.getMessage());
		builder.append(" :: method=").append(getBridgedMethod().toGenericString());
		builder.append(" :: invoked with handler type=").append(getBeanType().getName());
		
		if (args != null &&  args.length > 0) {
			builder.append(" and argument types ");
			for (int i = 0; i < args.length; i++) {
				String argClass = (args[i] != null) ? args[i].getClass().toString() : "null";
				builder.append(" : arg[").append(i).append("] ").append(argClass);
			}
		}
		else {
			builder.append(" and 0 arguments");
		}
		
		throw new IllegalArgumentException(builder.toString(), ex);
	}
	
	private void handleInvocationTargetException(InvocationTargetException ex) throws Exception {
		Throwable targetException = ex.getTargetException();
		if (targetException instanceof RuntimeException) {
			throw (RuntimeException) targetException;
		}
		if (targetException instanceof Error) {
			throw (Error) targetException;
		}
		if (targetException instanceof Exception) {
			throw (Exception) targetException;
		}
	}
	
}